#  Simple Map Editor
#  Andrew Ulysses Baker
#  September 2004
#  Public Domain


import random
import string
import sys
import os, os.path
import pygame
from pygame.locals import *

def load_image(name, colorkey=None):
    fullname = os.path.join('data', name)
    try:
        image = pygame.image.load(fullname)
    except pygame.error, message:
        print 'Cannot load image:', fullname
        raise SystemExit, message
    image = image.convert()
    if colorkey is not None:
        if colorkey is -1:
            colorkey = image.get_at((0,0))
        image.set_colorkey(colorkey, RLEACCEL)
    return image, image.get_rect()

def load_sound(name):
    class NoneSound:
        def play(self): pass
    if not pygame.mixer or not pygame.mixer.get_init():
        return NoneSound()
    fullname = os.path.join('data', name)
    try:
        sound = pygame.mixer.Sound(fullname)
    except pygame.error, message:
        print 'Cannot load sound:', fullname
        #raise SystemExit, message
        return NoneSound()
    return sound

def load_palette(directory, name, size):
    #Loads tile graphics in the following format:
    #Name\n
    palette = [[]] * size
    
    full_name = os.path.join(directory, name)
    if os.path.exists(full_name) == False:
        fnew = file(full_name, 'w')
        fnew.close()
        print "%s file created.  Fill it out..." % (full_name)
        #raise SystemExit
    fin = file(full_name, 'r')

    i=0
    whole = fin.read()
    lines = string.split(whole, '\n')
    for lin in lines:
        palette[i]=lin
        i+=1

    return palette

def load_world_ini(name):
    #Loads the world dimensions
    #MAP_DIMEN (assumes square maps)
    #TILE_DIMEN (assumes square tiles)

    #defaults
    MAPX = 0
    MAPY = 0

    if os.path.exists(name) == False:
        print "%s doesn't exist!"
        #raise SystemExit
    else:
        fin = file(name, 'r')

    whole = fin.read()
    lines = string.split(whole, '\n')

    for lin in lines:
        

        
        a, b = string.split(lin, ',')

        if a == "MAP_DIMEN":
            MAP_DIMEN = int(b)
        elif a == "TILE_DIMEN":
            TILE_DIMEN = int(b)
        elif a == "DEF_MAPX":
            MAPX = int(b)

        elif a == "DEF_MAPY":
            MAPY = int(b)
            

    return MAP_DIMEN, TILE_DIMEN, MAPX, MAPY

def load_map(map_tile_buffer, map_efx_buffer, mapx, mapy, map_dimen):

    full_name = os.path.join("maps", "m" + str(mapx) + "x" + str(mapy) + ".map")
    if os.path.exists(full_name) == False:
        print full_name
        fnew = file(full_name, 'w')
        for i in range(0, pow(map_dimen, 2)):
            map_tile_buffer[i]="black.bmp"
            map_efx_buffer[i]="None"
        save_map(map_tile_buffer, map_efx_buffer, mapx, mapy, map_dimen)
        fnew.close()

    print "Map: %s" % full_name
    fin = file(full_name, 'r')
    x=0
    y=0
    for y in range(0, map_dimen):
        line = fin.readline()
        cells = string.split(line)
        x=0
        for i in cells:
            elem = string.split(i, ',')
            #print x+(y*map_dimen)
            map_tile_buffer[x+(y*map_dimen)] = elem[0]
            map_efx_buffer[x+(y*map_dimen)] = elem[1]

            x+=1
        
    fin.close()
    return map_tile_buffer, map_efx_buffer

def save_map(map_tile_buffer, map_efx_buffer, mapx, mapy, map_dimen):
    full_name = os.path.join('maps', "m"+str(mapx)+"x"+str(mapy)+".map")
    fout = file(full_name,'w')

    n=0
    for j in range(0, pow(map_dimen, 2)):
        #print "saving... ",str(j)
        fout.write(map_tile_buffer[j] + ',' + map_efx_buffer[j] + " ")
        if n==map_dimen-1:
            n=0
            fout.write("\n")
        else:
            n+=1

    print "%s saved" % (full_name)
    fout.close()
    
def draw_map(map_tile_buffer, map_dimen, tile_dimen):
        
    for y in range(0, map_dimen):
        for x in range(0, map_dimen):
            #print str(x),',',str(y),',',map_tile_buffer[x+(y*map_dimen)]
            tile_image, tile_rect = load_image(map_tile_buffer[x+(y*map_dimen)], None)
            tile_rect.topleft = (x*tile_dimen, y*tile_dimen)

            background.blit(tile_image, tile_rect)

    screen.blit(background, (0,0))
        

def edit_screen(topleft, map_tile_buffer, map_efx_buffer, tile_palette, efx_palette, tile_index, efx_index):
    global map_dimen
    global tile_dimen
    
    x,y = topleft
    print x,y

    map_tile_buffer[x/tile_dimen+((y/tile_dimen)*map_dimen)] = tile_palette[tile_index]
    map_efx_buffer[x/tile_dimen+((y/tile_dimen)*map_dimen)] = efx_palette[efx_index]
    
    print map_tile_buffer[x/tile_dimen+((y/tile_dimen)*map_dimen)]
    return map_tile_buffer, map_efx_buffer

def palette_swap(index, palette):
    if index < len(palette):
        index += 1
    else:
        index = 0

    if palette[index]==[]:
        index = 0
        
    return index

def clear_text(text, locx, locy):
    global screen
    
    readout = font.render(text, 1, (0, 0, 0))
    screen.blit(readout, (locx, locy))

def update_text(text, locx, locy):
    global screen
    
    readout = font.render(text, 1, (200, 200, 200))
    screen.blit(readout, (locx, locy))


class Selector(pygame.sprite.Sprite):
    edit_state = 'tile'

    def __init__(self,map_dimen, tile_dimen):
        pygame.sprite.Sprite.__init__(self)
        screen = pygame.display.get_surface()
        self.image, self.rect = load_image('selector.png', -1)
        self.rect.width, self.rect.height = tile_dimen, tile_dimen
        self.x_velocity = 0
        self.y_velocity = 0

    def update(self, map_dimen, tile_dimen):
        global mapx
        global mapy

        new_map = False
        self.rect.move_ip(self.x_velocity, self.y_velocity)

        if self.rect.left < 0:
            self.rect.right = (map_dimen*tile_dimen)
            mapx -= 1
            new_map = True
            print mapx, mapy    
        elif self.rect.right > (map_dimen*tile_dimen):
            self.rect.left = 0
            mapx += 1
            new_map = True
            print mapx, mapy
        if self.rect.top < 0:
            self.rect.bottom = (map_dimen*tile_dimen)
            mapy -= 1
            new_map = True
            print mapx, mapy
        elif self.rect.bottom > (map_dimen*tile_dimen):
            self.rect.top = 0
            mapy += 1
            new_map = True
            print mapx, mapy
        return new_map

    def position(self):
        return self.rect.topleft

    def moveUp(self):
        self.y_velocity -= tile_dimen

    def moveDn(self):
        self.y_velocity += tile_dimen

    def moveLt(self):
        self.x_velocity -= tile_dimen

    def moveRt(self):
        self.x_velocity += tile_dimen
        
    
class Indicator(pygame.sprite.Sprite):

    def __init__(self, tile_palette, tile_index):
        global tile_dimen
        global map_dimen
        pygame.sprite.Sprite.__init__(self)
        screen = pygame.display.get_surface()
        self.image, self.rect = load_image(tile_palette[tile_index], None)
        self.rect.width, self.rect.height = tile_dimen, tile_dimen
        self.rect.topleft = (map_dimen*tile_dimen, 0)

    def update(self, tile_palette, tile_index):
        self.image, self.rect = load_image(tile_palette[tile_index], None)
        self.rect.width, self.rect.height = tile_dimen, tile_dimen
        self.rect.topleft = (map_dimen*tile_dimen, 0)
        
def main():

    #Initialize the configuration settings
    pygame.init()
    
    global screen
    global background
    global map_dimen
    global tile_dimen
    global mapx
    global mapy
    
    map_dimen, tile_dimen, mapx, mapy = load_world_ini('world.ini')

    tile_palette = load_palette('data', 'tile_palette.ini', 64)
    effect_palette = load_palette('data', 'effect_palette.ini', 8)

    #Initialize the buffers and stuff
    map_tile_buffer = [[]] * (map_dimen*map_dimen)
    map_efx_buffer = [[]] * (map_dimen*map_dimen)

    global tile_index 
    global efx_index 
    tile_index = 0
    efx_index = 0
    
    
    """load font"""
    global font, textpos
    font = pygame.font.Font(None, 36)

    
    screen = pygame.display.set_mode((map_dimen*tile_dimen+tile_dimen, map_dimen*tile_dimen+tile_dimen))
    pygame.display.set_caption('Simple Map Editor')

    
    #load the first map and create the background
    background = pygame.Surface(screen.get_size())
    background = background.convert()    
    map_tile_buffer, map_efx_buffer = load_map(map_tile_buffer, map_efx_buffer, mapx, mapy, map_dimen)
    draw_map(map_tile_buffer, map_dimen, tile_dimen)

    select = Selector(map_dimen, tile_dimen)
    selector = pygame.sprite.RenderClear(select)
    indicate = Indicator(tile_palette, tile_index)
    indicator = pygame.sprite.RenderClear(indicate)

    while 1:
        pygame.time.delay(40)

        
        for event in pygame.event.get():
            if event.type == QUIT:
                running = 0
            elif event.type == KEYDOWN:
                if event.key == K_ESCAPE:
                    running = 0
                elif event.key == K_LEFT:
                    select.moveLt()
                    clear_text(map_efx_buffer[temp_sx/tile_dimen+((temp_sy/tile_dimen)*map_dimen)], (map_dimen*tile_dimen)/2, map_dimen*tile_dimen)
                elif event.key == K_RIGHT:
                    select.moveRt()
                    clear_text(map_efx_buffer[temp_sx/tile_dimen+((temp_sy/tile_dimen)*map_dimen)], (map_dimen*tile_dimen)/2, map_dimen*tile_dimen)
                elif event.key == K_UP:
                    select.moveUp()
                    clear_text(map_efx_buffer[temp_sx/tile_dimen+((temp_sy/tile_dimen)*map_dimen)], (map_dimen*tile_dimen)/2, map_dimen*tile_dimen)
                elif event.key == K_DOWN:
                    select.moveDn()
                    clear_text(map_efx_buffer[temp_sx/tile_dimen+((temp_sy/tile_dimen)*map_dimen)], (map_dimen*tile_dimen)/2, map_dimen*tile_dimen)
                elif event.key == K_x:
                    map_tile_buffer, efx_tile_buffer = edit_screen(select.rect.topleft, map_tile_buffer, map_efx_buffer, tile_palette, effect_palette, tile_index, efx_index)
                    draw_map(map_tile_buffer, map_dimen, tile_dimen)
                elif event.key == K_c: #change tile
                    tile_index = palette_swap(tile_index, tile_palette)
                    indicate.update(tile_palette, tile_index)
                    
                elif event.key == K_v: #change effect
                    clear_text(effect_palette[efx_index], 0, map_dimen*tile_dimen)
                    efx_index = palette_swap(efx_index, effect_palette)
                    
                elif event.key == K_s:
                    save_map(map_tile_buffer, map_efx_buffer, mapx, mapy, map_dimen)
            elif event.type == KEYUP:
                if event.key == K_LEFT:
                    select.x_velocity = 0
                elif event.key == K_RIGHT:
                    select.x_velocity = 0
                elif event.key == K_UP:
                    select.y_velocity = 0
                elif event.key == K_DOWN:
                    select.y_velocity = 0

        temp_map_x = mapx
        temp_map_y = mapy
        
        selector.update(map_dimen, tile_dimen)

        if temp_map_x != mapx or temp_map_y != mapy:
            map_tile_buffer, map_efx_buffer = load_map(map_tile_buffer, map_efx_buffer, mapx, mapy, map_dimen)
            draw_map(map_tile_buffer, map_dimen, tile_dimen)

        update_text(effect_palette[efx_index], 0, map_dimen*tile_dimen)
        temp_sx, temp_sy = select.position()
        #print temp_sx/tile_dimen+((temp_sy/tile_dimen)*map_dimen)
        
        update_text(map_efx_buffer[temp_sx/tile_dimen+((temp_sy/tile_dimen)*map_dimen)], (map_dimen*tile_dimen)/2, map_dimen*tile_dimen)
        selector.clear(screen, background)
        selector.draw(screen)
        indicator.clear(screen, background)
        indicator.draw(screen)

        pygame.display.flip()
    """Exiting Main Loop"""

if __name__ == '__main__': main()

